=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2007-06-21 01:12:19 +0000
+++ bzrlib/graph.py	2007-06-21 01:58:29 +0000
@@ -278,17 +278,14 @@
                 return lca.pop()
             revisions = lca
 
-    def iter_topo(self, revisions):
+    def iter_topo_order(self, revisions):
         """Iterate through the input revisions in topological order.
 
         This sorting only ensures that parents come before their children.
         An ancestor may sort after a descendant if the relationship is not
         visible in the supplied list of revisions.
         """
-        subgraph = zip(revisions, self.get_parents(revisions))
-        subgraph = [(r, [p for p in ps if p in revisions])
-                    for r, ps in subgraph]
-        return tsort.topo_sort(subgraph)
+        return tsort.topo_sort(zip(revisions, self.get_parents(revisions)))
 
 
 class _BreadthFirstSearcher(object):

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2007-06-21 01:12:19 +0000
+++ bzrlib/tests/test_graph.py	2007-06-21 01:58:29 +0000
@@ -121,7 +121,7 @@
                     'rev3a': ['rev2a', 'rev2b'], 'rev3b': ['rev2b', 'rev2c']}
 
 
-class TestGraphWalker(TestCaseWithMemoryTransport):
+class TestGraph(TestCaseWithMemoryTransport):
 
     def make_graph(self, ancestors):
         tree = self.prepare_memory_tree('.')
@@ -281,10 +281,10 @@
         self.assertEqual([['rev4',], ['rev4']],
                          stacked.get_parents(['rev1', 'rev1']))
 
-    def test_iter_topo(self):
+    def test_iter_topo_order(self):
         graph = self.make_graph(ancestry_1)
         args = ['rev2a', 'rev3', 'rev1']
-        topo_args = list(graph.iter_topo(args))
+        topo_args = list(graph.iter_topo_order(args))
         self.assertEqual(set(args), set(topo_args))
         self.assertTrue(topo_args.index('rev2a') > topo_args.index('rev1'))
         self.assertTrue(topo_args.index('rev2a') < topo_args.index('rev3'))

=== modified file 'bzrlib/tests/test_tsort.py'
--- bzrlib/tests/test_tsort.py	2006-10-16 10:03:21 +0000
+++ bzrlib/tests/test_tsort.py	2007-06-21 01:58:29 +0000
@@ -85,6 +85,12 @@
                                    (8, [0, 1, 4, 5, 6])]),
                                   [0, 1, 2, 3, 4, 5, 6, 7, 8])
 
+    def test_tsort_unincluded_parent(self):
+        """Sort nodes, but don't include some parents in the output"""
+        self.assertSortAndIterate([(0, [1]),
+                                   (1, [2])],
+                                   [1, 0])
+
 
 class MergeSortTests(TestCase):
 

=== modified file 'bzrlib/tsort.py'
--- bzrlib/tsort.py	2007-04-19 00:03:01 +0000
+++ bzrlib/tsort.py	2007-06-21 01:58:29 +0000
@@ -61,6 +61,7 @@
         """
         # a dict of the graph.
         self._graph = dict(graph)
+        self._visitable = set(self._graph)
         ### if debugging:
         # self._original_graph = dict(graph)
         
@@ -120,6 +121,8 @@
                             # this parent was completed by a child on the
                             # call stack. skip it.
                             continue
+                        if next_node_name not in self._visitable:
+                            continue
                         # otherwise transfer it from the source graph into the
                         # top of the current depth first search stack.
                         try:

